package com.lzmispt.data.resource.service.file.impl;

package .tmp;

import com.lzmispt.core.entity.coreconst.CoreConst;
import com.lzmispt.core.service.authorize.IAccessToken;
import com.lzmispt.core.service.file.FileOperationClient;
import com.lzmispt.core.service.file.IFileChunkUploadService;
import com.lzmispt.core.service.file.config.FileServerConfig;
import com.lzmispt.data.center.entity.dto.databasestruct.DataSourceDTO;
import com.lzmispt.data.center.entity.dto.resource.apply.GetDataTableConnectionInfoVO;
import com.lzmispt.data.center.entity.enums.FileSysFieldEnum;
import com.lzmispt.data.center.entity.vo.resource.datasource.DataSourceVO;
import com.lzmispt.data.resource.client.DataApplyClient;
import com.lzmispt.data.resource.client.DataStandardClient;
import com.lzmispt.data.resource.dao.specialdatabase.IResSpecialDatabaseDao;
import com.lzmispt.data.resource.entity.dto.file.ChangeFileReferenceCountDTO;
import com.lzmispt.data.resource.entity.dto.resanddata.ResAndDataFileTableDTO;
import com.lzmispt.data.resource.manager.file.FileInfoManager;
import com.lzmispt.data.resource.service.file.IFTPFileService;
import com.lzmispt.dcbase.databasestruct.DataStructProvider;
import com.lzmispt.dcbase.databasestruct.TypeConverter;
import com.lzmispt.dcbase.dcconst.DCConst;
import com.lzmispt.dcbase.resfile.ResFileHelper;
import com.lzmispt.dcbase.sysfield.SysFieldHelper;
import com.lzmispt.dcbase.util.thumbnail.ExcelThumbnailUtil;
import com.lzmispt.dcbase.util.thumbnail.PdfThumbnailUtil;
import com.lzmispt.dcbase.util.thumbnail.PicThumbnailUtil;
import com.lzmispt.dcbase.util.thumbnail.PptThumbnailUtil;
import com.lzmispt.dcbase.util.thumbnail.WordThumbnailUtil;
import com.lzmispt.framework.entity.dto.file.FileAuthDTO;
import com.lzmispt.framework.entity.entity.ResultEntity;
import com.lzmispt.framework.exception.ExceptionUtil;
import com.lzmispt.framework.ftp.FTPUtil;
import com.lzmispt.framework.log.LogHelper;
import com.lzmispt.framework.utils.JedisUtil;
import com.lzmispt.framework.utils.WorkerId;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.Date;
import java.util.Map;
import java.util.Optional;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

@Service
@RefreshScope
public class FTPFileServiceImpl implements IFTPFileService {
  private static final String FILE_COUNT_NAME = "filecount";
  
  private static final String DOWNLOAD_TEMP_PATH = "/fileDownloadTemp/";
  
  private static final String FILE_PATH = System.getProperty("catalina.home") + "/webapps/file/resource";
  
  private static final Integer WIDTH = Integer.valueOf(400);
  
  private static final Integer HEIGHT = Integer.valueOf(200);
  
  private static final Integer NUM = Integer.valueOf(1024);
  
  @Autowired
  private IResSpecialDatabaseDao resSpecialDatabaseDao;
  
  @Autowired
  FileServerConfig fileServerConfig;
  
  @Autowired
  IAccessToken accessTokenService;
  
  @Autowired
  JedisUtil jedisUtil;
  
  @Autowired
  DataApplyClient applyClient;
  
  @Autowired
  DataStandardClient dataStandardClient;
  
  @Autowired
  IFileChunkUploadService chunkUploadService;
  
  public void getFileSize(HttpServletResponse response, Long fileTableId, Long fileId, Long authorizeId, String accessToken, boolean checkAuthority) {
    getFileSize(response, fileTableId, fileId, authorizeId, accessToken, checkAuthority, DCConst.DataResourceCategory.DATA);
  }
  
  public void getFileSize(HttpServletResponse response, Long fileTableId, Long fileId, Long authorizeId, String accessToken, boolean checkAuthority, Integer dataCategory) {
    if (checkAuthority) {
      if (!this.accessTokenService.isValidToken(accessToken))
        return; 
      FileAuthDTO fileAuth = (FileAuthDTO)this.jedisUtil.getObject(String.valueOf(fileId), FileAuthDTO.class);
      if (fileAuth == null || !authorizeId.equals(fileAuth.getAuthorizeId()))
        return; 
    } 
    Map<String, Object> ftpFileProps = getFTPFileProps(fileTableId, fileId, dataCategory);
    if (ftpFileProps == null)
      return; 
    String fullName = ftpFileProps.get(SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.FULL_NAME)).toString();
    String ftpPath = ftpFileProps.get(SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.PATH)).toString();
    Object serverModeObj = ftpFileProps.get(SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.SERVER_MODE));
    Integer serverMode = (serverModeObj == null) ? this.fileServerConfig.getDefaultFileServerMode() : Integer.valueOf(serverModeObj.toString());
    try (FileOperationClient dcFileOperationClient = this.chunkUploadService.getDCFileOperationClient(serverMode)) {
      FileOperationClient.FileInfo fileInfo = dcFileOperationClient.getFileInfo(ftpPath + "/" + fullName);
      if (fileInfo == null) {
        response.setStatus(404);
        return;
      } 
      response.setHeader("Content-Length", String.valueOf(fileInfo.getSize()));
      response.setStatus(200);
    } catch (Exception e) {
      LogHelper.error("获取文件服务器文件信息失败！", ExceptionUtil.getStackTraceMsg(e));
    } 
  }
  
  public void fileDownload(HttpServletRequest request, HttpServletResponse response, Long fileTableId, Long fileId, Long authorizeId, String accessToken) {
    fileDownload(request, response, fileTableId, fileId, authorizeId, accessToken, true);
  }
  
  public void fileDownload(HttpServletRequest request, HttpServletResponse response, Long fileTableId, Long fileId, Long authorizeId, String accessToken, boolean checkAuthority) {
    fileDownload(request, response, fileTableId, fileId, authorizeId, accessToken, checkAuthority, DCConst.DataResourceCategory.DATA, Boolean.valueOf(false));
  }
  
  public void fileDownloadForPreview(HttpServletRequest request, HttpServletResponse response, Long fileTableId, Long fileId, Long authorizeId, String accessToken, boolean checkAuthority, Integer dataCategory) {
    fileDownload(request, response, fileTableId, fileId, authorizeId, accessToken, checkAuthority, dataCategory, Boolean.valueOf(true));
  }
  
  public void fileDownload(HttpServletRequest request, HttpServletResponse response, Long fileTableId, Long fileId, Long authorizeId, String accessToken, boolean checkAuthority, Integer dataCategory, Boolean isPreview) {
    if (checkAuthority) {
      if (!this.accessTokenService.isValidToken(accessToken))
        return; 
      FileAuthDTO fileAuth = (FileAuthDTO)this.jedisUtil.getObject(String.valueOf(fileId), FileAuthDTO.class);
      if (fileAuth == null || !authorizeId.equals(fileAuth.getAuthorizeId()))
        return; 
    } 
    try {
      Map<String, Object> ftpFileProps = getFTPFileProps(fileTableId, fileId, dataCategory);
      if (ftpFileProps == null)
        return; 
      String ext = Optional.<T>ofNullable((T)ftpFileProps.get(SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.EXT))).orElse((T)"").toString();
      String attachmentFileName = ftpFileProps.get(SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.NAME)).toString();
      if (!StringUtils.isEmpty(ext) && !attachmentFileName.contains("."))
        attachmentFileName = attachmentFileName + "." + ext; 
      Object serverModeObj = ftpFileProps.get(SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.SERVER_MODE));
      Integer serverMode = (serverModeObj == null) ? this.fileServerConfig.getDefaultFileServerMode() : Integer.valueOf(serverModeObj.toString());
      RangeConfig rangeConfig = downloadFile(request, response, ftpFileProps.get(SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.PATH)).toString(), ftpFileProps
          .get(SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.FULL_NAME)).toString(), attachmentFileName, serverMode);
    } catch (Exception e) {
      LogHelper.error("FTP文件下载失败！", ExceptionUtil.getStackTraceMsg(e));
    } 
  }
  
  public ResultEntity<Boolean> changeFileReferenceCount(ChangeFileReferenceCountDTO changeFileReferenceCountDTO, String platCode) {
    synchronized (changeFileReferenceCountDTO.getFileId()) {
      Long specialDatabaseId = changeFileReferenceCountDTO.getSpecialDatabaseId();
      ResFileHelper.updateFileDataByFileId(changeFileReferenceCountDTO.getFileId(), changeFileReferenceCountDTO.getReferenceCount(), platCode, specialDatabaseId);
      return ResultEntity.successMessage(Boolean.valueOf(true));
    } 
  }
  
  public void generateThumbnail(Long fileTableId, Long fileId, Integer dataCategory, String tokenId) {
    Map<String, Object> ftpFileProps = getFTPFileProps(fileTableId, fileId, dataCategory);
    if (ftpFileProps == null)
      return; 
    String ext = Optional.<T>ofNullable((T)ftpFileProps.get(SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.EXT))).orElse((T)"").toString();
    if (!isGenerateThumbnail(ext))
      return; 
    Object serverModeObj = ftpFileProps.get(SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.SERVER_MODE));
    Integer serverMode = (serverModeObj == null) ? this.fileServerConfig.getDefaultFileServerMode() : Integer.valueOf(serverModeObj.toString());
    String remoteFilePath = (new StringBuilder()).append(ftpFileProps.get(SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.PATH))).append("/").append(ftpFileProps.get(SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.FULL_NAME)).toString()).toString();
    String localFilePath = FILE_PATH + "/" + ftpFileProps.get(SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.FULL_NAME)).toString();
    String localThumbnailFilePath = FILE_PATH + "/" + ftpFileProps.get(SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.FULL_NAME)).toString().split("\\.")[0] + "_thumbnail.png";
    String remoteTargetFilePath = ftpFileProps.get(SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.PATH)).toString() + "/" + ftpFileProps.get(SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.FULL_NAME)).toString().split("\\.")[0] + "_thumbnail.png";
    try (FileOperationClient dcFileOperationClient = this.chunkUploadService.getDCFileOperationClient(serverMode)) {
      dcFileOperationClient.download(remoteFilePath, localFilePath);
      buildThumbnail(ext, localFilePath, localThumbnailFilePath, tokenId);
      FileInputStream fileInputStream = new FileInputStream(new File(localThumbnailFilePath));
      dcFileOperationClient.uploadFile(remoteTargetFilePath, 0L, fileInputStream);
      fileInputStream.close();
    } catch (Exception e) {
      LogHelper.error("FTP文件下载失败！", ExceptionUtil.getStackTraceMsg(e));
    } finally {
      File file = new File(localFilePath);
      file.delete();
      File fileThumbnail = new File(localThumbnailFilePath);
      fileThumbnail.delete();
    } 
  }
  
  public void receivePdfFromKKFile(MultipartFile file, String tokenId) {
    try {
      if (file.isEmpty()) {
        LogHelper.error("接收kkFile传入文件异常！", "文件为空！");
        return;
      } 
      String f = file.getOriginalFilename();
      f = URLDecoder.decode(f, "utf-8");
      LogHelper.info("接收kkFile传入文件生成路径：" + FILE_PATH + "/" + f);
      try(InputStream inputStream = file.getInputStream(); 
          OutputStream outputStream = new FileOutputStream(FILE_PATH + "/" + f)) {
        byte[] buffer = new byte[NUM.intValue()];
        int bytesRead;
        while ((bytesRead = inputStream.read(buffer)) != -1)
          outputStream.write(buffer, 0, bytesRead); 
      } catch (IOException e) {
        LogHelper.error("接收kkFile传入文件异常！", ExceptionUtil.getStackTraceMsg(e));
      } 
    } catch (Exception e) {
      LogHelper.error("接收kkFile传入文件异常！", ExceptionUtil.getStackTraceMsg(e));
    } 
  }
  
  private void buildThumbnail(String fileExt, String localFilePath, String localThumbnailFilePath, String tokenId) {
    if (isImageFile(fileExt))
      PicThumbnailUtil.generateThumbnail(localFilePath, localThumbnailFilePath); 
    if ("doc".equals(fileExt) || "docx".equals(fileExt)) {
      WordThumbnailUtil.generateThumbnail(localFilePath, localThumbnailFilePath, tokenId);
    } else if ("xls".equals(fileExt) || "xlsx".equals(fileExt)) {
      ExcelThumbnailUtil.generateThumbnail(localFilePath, localThumbnailFilePath, tokenId);
    } else if ("ppt".equals(fileExt) || "pptx".equals(fileExt)) {
      PptThumbnailUtil.generateThumbnail(localFilePath, localThumbnailFilePath, tokenId);
    } else if ("pdf".equals(fileExt)) {
      PdfThumbnailUtil.generateThumbnail(localFilePath, localThumbnailFilePath);
    } 
  }
  
  private boolean isGenerateThumbnail(String fileExt) {
    if (fileExt == null || fileExt.isEmpty())
      return false; 
    if (isImageFile(fileExt))
      return true; 
    if ("doc".equals(fileExt) || "docx".equals(fileExt))
      return true; 
    if ("xls".equals(fileExt) || "xlsx".equals(fileExt))
      return true; 
    if ("ppt".equals(fileExt) || "pptx".equals(fileExt))
      return true; 
    if ("pdf".equals(fileExt))
      return true; 
    return false;
  }
  
  private boolean isImageFile(String fileExt) {
    if (fileExt == null || fileExt.isEmpty())
      return false; 
    if (fileExt != null) {
      String lowercaseExtension = fileExt.toLowerCase();
      return (lowercaseExtension.equals("jpg") || lowercaseExtension
        .equals("jpeg") || lowercaseExtension
        .equals("png") || lowercaseExtension
        .equals("gif") || lowercaseExtension
        .equals("bmp"));
    } 
    return false;
  }
  
  public void fileDownload(HttpServletResponse response, String relativePath, String fileFullName, String originName) {
    if (StringUtils.isEmpty(fileFullName))
      return; 
    if (StringUtils.isEmpty(originName))
      originName = fileFullName; 
    File file = null;
    String tempFilePath = this.fileServerConfig.getTempFilePath() + "/fileDownloadTemp/" + fileFullName;
    try {
      file = FTPUtil.downloadFile(this.fileServerConfig.getFtp().getHost(), this.fileServerConfig.getFtp().getPort().intValue(), this.fileServerConfig.getFtp().getUsername(), this.fileServerConfig.getFtp().getPassword(), relativePath, fileFullName, tempFilePath);
      download(response, file, originName);
    } catch (IOException e) {
      LogHelper.error("FTP文件下载失败！relativePath：" + relativePath + ",fileFUllName：" + fileFullName, ExceptionUtil.getStackTraceMsg(e));
    } finally {
      if (file != null) {
        boolean delete = file.delete();
        if (!delete)
          LogHelper.error("缓存文件删除失败！位置：" + tempFilePath); 
      } 
    } 
  }
  
  public RangeConfig downloadFile(HttpServletRequest request, HttpServletResponse response, String path, String fileName, String originName, Integer serverMode) throws IOException {
    try (FileOperationClient dcFileOperationClient = this.chunkUploadService.getDCFileOperationClient(serverMode)) {
      FileOperationClient.FileInfo fileInfo = dcFileOperationClient.getFileInfo(path + "/" + fileName);
      if (fileInfo == null) {
        response.setStatus(404);
        return null;
      } 
      RangeConfig rangeConfig = getRangeAndHandleResponseSetting(request, response, fileInfo.getSize().longValue(), originName);
      if (rangeConfig == null) {
        response.setStatus(416);
        return null;
      } 
      dcFileOperationClient.transferRemoteFile(fileInfo
          .getPath(), rangeConfig
          .getStartByte(), inputStream -> transfer(inputStream, rangeConfig.getContentLength(), (OutputStream)response.getOutputStream()));
      return rangeConfig;
    } catch (Exception e) {
      LogHelper.error("下载文件异常", ExceptionUtil.getStackTraceMsg(e));
    } finally {
      try {
        if (response.getOutputStream() != null)
          response.getOutputStream().close(); 
      } catch (Exception e) {
        LogHelper.error("关闭输入输出流失败", ExceptionUtil.getStackTraceMsg(e));
      } 
    } 
    return null;
  }
  
  public static RangeConfig getRangeAndHandleResponseSetting(HttpServletRequest request, HttpServletResponse response, long fileSize, String fileName) {
    RangeConfig rangeConfig = getRangeConfig(request.getHeader("Range"), fileSize);
    long startByte = rangeConfig.getStartByte();
    long endByte = rangeConfig.getEndByte();
    long contentLength = rangeConfig.getContentLength();
    if (startByte >= endByte || startByte < 0L || endByte > fileSize - 1L)
      return null; 
    Integer httpStatus = Integer.valueOf((startByte == 0L && endByte == fileSize - 1L) ? 200 : 206);
    String contentType = request.getServletContext().getMimeType(fileName);
    byte[] fileNameBytes = fileName.getBytes(StandardCharsets.UTF_8);
    fileName = new String(fileNameBytes, 0, fileNameBytes.length, StandardCharsets.ISO_8859_1);
    String contentRange = "bytes " + startByte + "-" + endByte + "/" + fileSize;
    setResponsePartialConfig(response, fileName, httpStatus, contentLength, contentType, contentRange);
    return rangeConfig;
  }
  
  private static RangeConfig getRangeConfig(String range, long fileSize) {
    long startByte = 0L;
    long endByte = fileSize - 1L;
    long contentLength = fileSize;
    if (range != null && range.contains("bytes=") && range.contains("-")) {
      range = range.substring(range.lastIndexOf("=") + 1).trim();
      String[] ranges = range.split("-");
      try {
        if (ranges.length == 1) {
          if (range.startsWith("-")) {
            endByte = Long.parseLong(ranges[0]);
          } else if (range.endsWith("-")) {
            startByte = Long.parseLong(ranges[0]);
          } 
        } else if (ranges.length == 2) {
          startByte = Long.parseLong(ranges[0]);
          endByte = Long.parseLong(ranges[1]);
        } 
        contentLength = endByte - startByte + 1L;
      } catch (NumberFormatException numberFormatException) {}
    } 
    return new RangeConfig(startByte, endByte, contentLength);
  }
  
  private static void setResponsePartialConfig(HttpServletResponse response, String originName, Integer httpStatus, long contentLength, String contentType, String contentRange) {
    response.setHeader("Accept-Ranges", "bytes");
    response.setStatus(httpStatus.intValue());
    response.setContentType(contentType);
    response.setHeader("Content-Type", (contentType == null) ? "application/octet-stream" : contentType);
    response.setHeader("Content-Disposition", "inline;filename=" + originName);
    response.setHeader("Content-Length", String.valueOf(contentLength));
    response.setHeader("Content-Range", contentRange);
  }
  
  public static void transfer(InputStream inputStream, long length, OutputStream out) throws IOException {
    byte[] buffer = new byte[Math.toIntExact(Math.min(length, CoreConst.FILE_SIZE_UNIT.intValue()))];
    int read = 0;
    while (length > 0L) {
      read = inputStream.read(buffer);
      if (read == -1)
        break; 
      if (read < buffer.length)
        buffer = Arrays.copyOf(buffer, read); 
      out.write(buffer, 0, read);
      length -= buffer.length;
      if (length > 0L && length < buffer.length)
        buffer = new byte[Math.toIntExact(length)]; 
    } 
  }
  
  public ResultEntity<Long> fileAuthorize(Long fileTableId, Long fileId) {
    return fileAuthorize(fileId);
  }
  
  public ResultEntity<Long> fileAuthorize(Long fileId) {
    FileAuthDTO fileAuth = (FileAuthDTO)this.jedisUtil.getObject(String.valueOf(fileId), FileAuthDTO.class);
    if (fileAuth == null) {
      Long authorizeId = (Long)WorkerId.getNextId(Long.class);
      fileAuth = new FileAuthDTO(new Date(), authorizeId);
    } 
    this.jedisUtil.addObject(String.valueOf(fileId), fileAuth, this.fileServerConfig.getFileDownloadExpireTime().intValue());
    return ResultEntity.successMessage(fileAuth.getAuthorizeId());
  }
  
  public ResultEntity<Boolean> fileAuthorizeCheck(Long fileId, Long authorizeId) {
    FileAuthDTO fileAuth = (FileAuthDTO)this.jedisUtil.getObject(String.valueOf(fileId), FileAuthDTO.class);
    if (fileAuth == null || !authorizeId.equals(fileAuth.getAuthorizeId()))
      return ResultEntity.successMessage(Boolean.valueOf(false)); 
    return ResultEntity.successMessage(Boolean.valueOf(true));
  }
  
  public ResultEntity<Long> getFileCount(Long fileTableId, Integer dataCategory, String type) {
    Map<String, Object> fileCountMap = getFTPFileProps(fileTableId, null, dataCategory, type);
    if (fileCountMap != null && fileCountMap.containsKey("filecount") && fileCountMap.get("filecount") != null) {
      Long count = Long.valueOf(Long.parseLong(fileCountMap.get("filecount").toString()));
      return ResultEntity.successMessage(count);
    } 
    return ResultEntity.successMessage(Long.valueOf(0L));
  }
  
  private Map<String, Object> getFTPFileProps(Long fileTableId, Long fileId, Integer dataCategory) {
    return getFTPFileProps(fileTableId, fileId, dataCategory, null);
  }
  
  private Map<String, Object> getFTPFileProps(Long fileTableId, Long fileId, Integer dataCategory, String type) {
    String platCode = "Manage";
    ResAndDataFileTableDTO fileTable = FileInfoManager.getFileTableById(fileTableId, platCode, dataCategory);
    if (fileTable == null) {
      platCode = "Collection";
      fileTable = FileInfoManager.getFileTableById(fileTableId, platCode, dataCategory);
    } 
    if (fileTable == null) {
      platCode = "Apply";
      fileTable = FileInfoManager.getFileTableById(fileTableId, platCode, dataCategory);
    } 
    DataSourceVO dataSourceVO = null;
    if (fileTable != null && fileTable.getDataSourceId() != null)
      dataSourceVO = (DataSourceVO)this.dataStandardClient.getDataSourceInfoDoNotDecryptByIdNotTokenId(fileTable.getDataSourceId(), platCode).getObj(); 
    if (dataSourceVO == null)
      return null; 
    DataSourceDTO dataSourceDTO = new DataSourceDTO();
    BeanUtils.copyProperties(dataSourceVO, dataSourceDTO);
    ResultEntity<DataStructProvider> getDataBaseProviderRet = DataStructProvider.getDataBaseProvider(dataSourceDTO);
    if (!getDataBaseProviderRet.getCode().equals(Integer.valueOf(0)))
      return null; 
    Map<String, Object> result = null;
    if (type == null) {
      result = selectFileProp(fileId, fileTable.getFileTableName(), (DataStructProvider)getDataBaseProviderRet.getObj());
    } else if (type.equalsIgnoreCase("today")) {
      result = selectTodayFileCount(fileTable.getFileTableName(), (DataStructProvider)getDataBaseProviderRet.getObj());
    } else {
      result = selectAllFileCount(fileTable.getFileTableName(), (DataStructProvider)getDataBaseProviderRet.getObj());
    } 
    if (result != null) {
      result.put("platCode", platCode);
      result.put("directoryId", fileTable.getDirectoryId());
    } 
    return result;
  }
  
  private Map<String, Object> selectFileProp(Long fileId, String tableName, DataStructProvider dataStructProvider) {
    String sql = "select * from " + TypeConverter.getColumnSqlStr(tableName, dataStructProvider.getDbType()) + " where " + TypeConverter.getColumnSqlStr(SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.ID), dataStructProvider.getDbType()) + " = " + fileId;
    return dataStructProvider.selectOneQuery(sql);
  }
  
  private Map<String, Object> selectAllFileCount(String tableName, DataStructProvider dataStructProvider) {
    String aliasName = TypeConverter.getColumnSqlStr("filecount", dataStructProvider.getDbType());
    String sql = "select count(*) " + aliasName + " from " + TypeConverter.getColumnSqlStr(tableName, dataStructProvider.getDbType()) + " where " + TypeConverter.getColumnSqlStr(SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.IS_DELETE), dataStructProvider.getDbType()) + " = 0  and " + TypeConverter.getColumnSqlStr(SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.NAME), dataStructProvider.getDbType()) + " is not null";
    return dataStructProvider.selectOneQuery(sql);
  }
  
  private Map<String, Object> selectTodayFileCount(String tableName, DataStructProvider dataStructProvider) {
    String aliasName = TypeConverter.getColumnSqlStr("filecount", dataStructProvider.getDbType());
    String sql = "select count(*) " + aliasName + " from " + TypeConverter.getColumnSqlStr(tableName, dataStructProvider.getDbType()) + " where " + TypeConverter.getColumnSqlStr(SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.IS_DELETE), dataStructProvider.getDbType()) + " = 0  and " + TypeConverter.getColumnSqlStr(SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.NAME), dataStructProvider.getDbType()) + " is not null";
    String startTimeCol = TypeConverter.getColumnSqlStr(SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.START_TIME), dataStructProvider.getDbType());
    if (dataStructProvider.getDbType().equalsIgnoreCase("mysql")) {
      sql = sql + " and DATE(" + startTimeCol + ") = CURDATE()";
    } else if (dataStructProvider.getDbType().equalsIgnoreCase("sqlserver")) {
      sql = sql + " and CONVERT(DATE, " + startTimeCol + ") = CAST(GETDATE() as date)";
    } else if (dataStructProvider.getDbType().equalsIgnoreCase("oracle")) {
      sql = sql + " and " + startTimeCol + ">= TRUNC(SYSDATE) AND " + startTimeCol + " < TRUNC(SYSDATE) + 1";
    } else if (dataStructProvider.getDbType().equalsIgnoreCase("postgresql") || dataStructProvider.getDbType().equalsIgnoreCase("kingbase")) {
      sql = sql + " and " + startTimeCol + "::date = CURRENT_DATE";
    } else if (dataStructProvider.getDbType().equalsIgnoreCase("dm")) {
      sql = sql + " and to_char(" + startTimeCol + ",'YYYY-MM-DD')= CURDATE()";
    } 
    return dataStructProvider.selectOneQuery(sql);
  }
  
  private Map<String, Object> getApplyFTPFileProps(Long fileTableId, Long fileId) {
    try {
      ResultEntity<GetDataTableConnectionInfoVO> connectionInfoResult = this.applyClient.getFileTableConnectionInfo(fileTableId, "0");
      if (connectionInfoResult == null || connectionInfoResult.getObj() == null)
        return null; 
      GetDataTableConnectionInfoVO connectionInfoVO = (GetDataTableConnectionInfoVO)connectionInfoResult.getObj();
      GetDataTableConnectionInfoVO.DataSourceInfo dataSourceInfo = connectionInfoVO.getDataSourceInfo();
      DataSourceDTO dataSourceDTO = new DataSourceDTO();
      dataSourceDTO.setType(dataSourceInfo.getDbType());
      dataSourceDTO.setUser(dataSourceInfo.getUsername());
      dataSourceDTO.setDbSchema(dataSourceInfo.getDbSchema());
      dataSourceDTO.setDbName(connectionInfoVO.getDbName());
      dataSourceDTO.setPort(dataSourceInfo.getPort());
      dataSourceDTO.setIpName(dataSourceInfo.getDataSourceUrl());
      dataSourceDTO.setPassword(dataSourceInfo.getPassword());
      ResultEntity<DataStructProvider> getDataBaseProviderRet = DataStructProvider.getDataBaseProvider(dataSourceDTO, true, false);
      if (!getDataBaseProviderRet.getCode().equals(Integer.valueOf(0)))
        return null; 
      return selectFileProp(fileId, connectionInfoVO.getTableName(), (DataStructProvider)getDataBaseProviderRet.getObj());
    } catch (Exception e) {
      LogHelper.error("应用平台获取文件信息失败", ExceptionUtil.getStackTraceMsg(e));
      return null;
    } 
  }
  
  private void download(HttpServletResponse response, File file, String fileName) {
    InputStream in = null;
    OutputStream out = null;
    try {
      in = new BufferedInputStream(new FileInputStream(file));
      byte[] buffer = new byte[in.available()];
      int read = in.read(buffer);
      if (read <= 0)
        return; 
      response.reset();
      response.addHeader("Content-Disposition", "attachment;filename=" + new String(fileName.getBytes(StandardCharsets.UTF_8), StandardCharsets.ISO_8859_1));
      response.addHeader("Content-Length", "" + file.length());
      out = new BufferedOutputStream((OutputStream)response.getOutputStream());
      response.setContentType("application/octet-stream");
      out.write(buffer);
      out.flush();
    } catch (Exception e) {
      LogHelper.error("获取文件失败", ExceptionUtil.getStackTraceMsg(e));
    } finally {
      if (in != null)
        try {
          in.close();
        } catch (IOException e) {
          LogHelper.error("文件下载时输入流关闭失败", ExceptionUtil.getStackTraceMsg(e));
        }  
      if (out != null)
        try {
          out.close();
        } catch (IOException e) {
          LogHelper.error("文件下载时输出流关闭失败", ExceptionUtil.getStackTraceMsg(e));
        }  
    } 
  }
}
